@@ -98,9 +98,10 @@ method above.  The *end_point* parameter specifies the base url to the server th
 """
 import inspect
 import json
+import sys
 from abc import ABC
 from functools import wraps
-from typing import Any, Dict, TypeVar, Optional, Type, AsyncIterator, Generic, Mapping, Iterator
+from typing import Any, Dict, TypeVar, Optional, Type, Generic, Mapping, Iterator
 
 import aiohttp
 
@@ -112,6 +113,17 @@ __all__ = ["WebInterface"]
 C = TypeVar('C', bound="WebInterface")
 
 
+class InvocationError(Exception):
+    def __init__(self, message: str):
+        self.message = message
+
+    def __str__(self):
+        return self.message
+
+    def __repr__(self):
+        return self.message
+
+
 # noinspection PyProtectedMember
 class WebInterface(ABC):
 
@@ -144,6 +156,8 @@ class WebInterface(ABC):
                 arg_spec = inspect.getfullargspec(api._func)
             except Exception:
                 arg_spec = inspect.getfullargspec(api._func.__func__)
+            if len(arg_spec) - 1 < len(args):
+                raise TypeError(f"Too many arguments supplied in call to {api.name}")
             if arg_spec.varargs is None:
                 kwargs_.update({
                     arg_spec.args[n + 1]: arg for n, arg in enumerate(args)
@@ -152,33 +166,43 @@ class WebInterface(ABC):
                 kwargs_[arg_spec.varargs] = args
             # noinspection PyBroadException
             rest_method = api._func._bantam_web_api.method
-            if rest_method.value == RestMethod.GET.value:
-                url_args = cls._generate_url_args(kwargs=kwargs_)
-                url = f"{base_url}{url_args}"
+            data = None
+            resp = None
+            try:
                 async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
-                    async with session.get(url) as resp:
-                        resp.raise_for_status()
-                        data = (await resp.content.read()).decode('utf-8')
-                        if api.is_constructor:
-                            if hasattr(clazz, 'jsonrepr'):
-                                repr_ = clazz.jsonrepr(data)
-                                self_id = repr_[api.uuid_param or 'uuid']
-                            else:
-                                repr_ = json.loads(data)
-                                self_id = repr_[api.uuid_param or 'uuid']
-                            return cls_(self_id)
-                        return conversions.from_str(data, api.return_type)
-            else:
-                payload = json.dumps({conversions.to_str(k): conversions.normalize_to_json_compat(v)
-                                      for k, v in kwargs_.items()})
-                async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
-                    async with session.post(base_url, data=payload) as resp:
-                        resp.raise_for_status()
-                        data = (await resp.content.read()).decode('utf-8')
-                        if api.is_constructor:
-                            self_id = json.loads(data)[api.uuid_param or 'uuid']
-                            return cls_(self_id)
-                        return conversions.from_str(data, api.return_type)
+                    if rest_method.value == RestMethod.GET.value:
+                        url_args = cls._generate_url_args(kwargs=kwargs_)
+                        url = f"{base_url}{url_args}"
+                        async with session.get(url) as resp:
+                            data = (await resp.content.read()).decode('utf-8')
+                            if not resp.ok:
+                                sys.stderr.write(data + '\n')
+                            resp.raise_for_status()
+                            if api.is_constructor:
+                                if hasattr(clazz, 'jsonrepr'):
+                                    repr_ = clazz.jsonrepr(data)
+                                    self_id = repr_[api.uuid_param or 'uuid']
+                                else:
+                                    repr_ = json.loads(data)
+                                    self_id = repr_[api.uuid_param or 'uuid']
+                                return cls_(self_id)
+                            return conversions.from_str(data, api.return_type)
+                    else:
+                        payload = json.dumps({conversions.to_str(k): conversions.normalize_to_json_compat(v)
+                                              for k, v in kwargs_.items()})
+                        async with session.post(base_url, data=payload) as resp:
+                            data = (await resp.content.read()).decode('utf-8')
+                            if not resp.ok:
+                                sys.stderr.write(data + '\n')
+                            resp.raise_for_status()
+                            if api.is_constructor:
+                                self_id = json.loads(data)[api.uuid_param or 'uuid']
+                                return cls_(self_id)
+                            return conversions.from_str(data, api.return_type)
+            except aiohttp.ClientResponseError as e:
+                error_body = data if resp is not None else "<<no response text/traceback info>>"
+                error_body += f"\n\nRequest to {api.name} failed: {e.message}"
+                raise InvocationError(error_body)
 
         setattr(clazz, name, class_method)
 
@@ -199,7 +223,8 @@ class WebInterface(ABC):
         # noinspection PyDecorator,PyUnusedLocal
         @classmethod
         async def class_method_streamed(cls_, *args, **kwargs):
-            nonlocal api, end_point
+            nonlocal api, end_point, common_headers
+            resp = None
             # noinspection PyBroadException
             try:
                 arg_spec = inspect.getfullargspec(api._func)
@@ -209,40 +234,49 @@ class WebInterface(ABC):
                 arg_spec.args[n + 1]: arg for n, arg in enumerate(args)
             })
             rest_method = api._func._bantam_web_api.method
-            if rest_method.value == RestMethod.GET.value:
-                url_args = cls._generate_url_args(kwargs=kwargs)
-                url = f"{base_url}{url_args}"
-                async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
-                    async with session.get(url) as resp:
-                        resp.raise_for_status()
-                        buffer = ""
-                        async for data, _ in resp.content.iter_chunks():
-                            if data:
-                                if api.return_type != bytes:
-                                    buffer += data.decode('utf-8')
-                                    *data_items, buffer = buffer.split('\0')
-                                    for datum in data_items:
-                                        yield conversions.from_str(datum, api.return_type)
-                                else:
-                                    data = data.decode('utf-8')
-                                    yield conversions.from_str(data, api.return_type)
-            else:
-                payload = json.dumps({conversions.to_str(k): conversions.normalize_to_json_compat(v)
-                                      for k, v in kwargs.items()})
-                async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
-                    async with session.post(base_url, data=payload) as resp:
-                        buffer = ""
-                        async for data, _ in resp.content.iter_chunks():
+            common_headers = (common_headers or {}).update({'Content-Type': 'text/streamed'})
+            try:
+                if rest_method.value == RestMethod.GET.value:
+                    url_args = cls._generate_url_args(kwargs=kwargs)
+                    url = f"{base_url}{url_args}"
+                    async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
+                        async with session.get(url) as resp:
+                            if not resp.ok:
+                                sys.stderr.write((await resp.content.read()).decode('utf-8') + '\n')
                             resp.raise_for_status()
-                            if data:
-                                if api.return_type != bytes:
-                                    buffer += data.decode('utf-8')
-                                    *data_items, buffer = buffer.split('\0')
-                                    for datum in data_items:
-                                        yield conversions.from_str(datum, api.return_type)
-                                else:
-                                    data = data.decode('utf-8')
-                                    yield conversions.from_str(data, api.return_type)
+                            buffer = ""
+                            async for data, _ in resp.content.iter_chunks():
+                                if data:
+                                    if api.return_type != bytes:
+                                        buffer += data.decode('utf-8')
+                                        *data_items, buffer = buffer.split('\0')
+                                        for datum in data_items:
+                                            yield conversions.from_str(datum, api.return_type)
+                                    else:
+                                        data = data.decode('utf-8')
+                                        yield conversions.from_str(data, api.return_type)
+                else:
+                    payload = json.dumps({conversions.to_str(k): conversions.normalize_to_json_compat(v)
+                                          for k, v in kwargs.items()})
+                    async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
+                        async with session.post(base_url, data=payload) as resp:
+                            if not resp.ok:
+                                sys.stderr.write((await resp.content.read()).decode('utf-8') + '\n')
+                            resp.raise_for_status()
+                            buffer = ""
+                            async for data, _ in resp.content.iter_chunks():
+                                if data:
+                                    if api.return_type != bytes:
+                                        buffer += data.decode('utf-8')
+                                        *data_items, buffer = buffer.split('\0')
+                                        for datum in data_items:
+                                            yield conversions.from_str(datum, api.return_type)
+                                    else:
+                                        data = data.decode('utf-8')
+                                        yield conversions.from_str(data, api.return_type)
+            except aiohttp.ClientResponseError as e:
+                body = e.message
+                raise Exception(body)
 
         setattr(clazz, name, class_method_streamed)
 
@@ -270,26 +304,35 @@ class WebInterface(ABC):
             })
             # noinspection PyBroadException
             rest_method = api._func._bantam_web_api.method
-            if rest_method.value == RestMethod.GET.value:
-                url_args = cls._generate_url_args(self_id=self.self_id, kwargs=kwargs_)
-                url = f"{base_url}{url_args}"
-                async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
-                    async with session.get(url) as resp:
-                        resp.raise_for_status()
-                        data = (await resp.content.read()).decode('utf-8')
-                        return conversions.from_str(data, api.return_type)
-            else:
-                kwargs_['self'] = self.self_id
-                payload = json.dumps({conversions.to_str(k): conversions.normalize_to_json_compat(v)
-                                      for k, v in kwargs_.items()})
-                async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
-                    async with session.post(base_url, data=payload) as resp:
-                        resp.raise_for_status()
-                        data = (await resp.content.read()).decode('utf-8')
-                        if api.is_constructor:
-                            self_id = json.loads(data)['self_id']
-                            return clazz(self_id)
-                        return conversions.from_str(data, api.return_type)
+            try:
+                if rest_method.value == RestMethod.GET.value:
+                    url_args = cls._generate_url_args(self_id=self.self_id, kwargs=kwargs_)
+                    url = f"{base_url}{url_args}"
+                    async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
+                        async with session.get(url) as resp:
+                            if not resp.ok:
+                                sys.stderr.write((await resp.content.read()).decode('utf-8') + '\n')
+                            resp.raise_for_status()
+                            data = (await resp.content.read()).decode('utf-8')
+                            return conversions.from_str(data, api.return_type)
+                else:
+                    kwargs_['self'] = self.self_id
+                    payload = json.dumps({conversions.to_str(k): conversions.normalize_to_json_compat(v)
+                                          for k, v in kwargs_.items()})
+                    async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
+                        async with session.post(base_url, data=payload) as resp:
+                            if not resp.ok:
+                                sys.stderr.write((await resp.content.read()).decode('utf-8') + '\n')
+                            resp.raise_for_status()
+                            data = (await resp.content.read()).decode('utf-8')
+                            if api.is_constructor:
+                                self_id = json.loads(data)['self_id']
+                                return clazz(self_id)
+                            return conversions.from_str(data, api.return_type)
+            except aiohttp.ClientResponseError as e:
+                body = resp.content if resp is not None else "<<no response text/traceback info>>"
+                body += f"\n\nRequest to {api.name} failed: {e.message}"
+                raise Exception(body)
 
         setattr(clazz, name, instance_method)
 
@@ -310,41 +353,51 @@ class WebInterface(ABC):
                 arg_spec.args[n + 1]: arg for n, arg in enumerate(args)
             })
             rest_method = api.method
-            if rest_method == RestMethod.GET:
-                url_args = cls._generate_url_args(self_id=self.self_id, kwargs=kwargs_)
-                url = f"{base_url}{url_args}"
-                async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
-                    async with session.get(url) as resp:
-                        resp.raise_for_status()
-                        buffer = ""
-                        async for data, _ in resp.content.iter_chunks():
-                            if data:
-                                if api.return_type != bytes:
-                                    buffer += data.decode('utf-8')
-                                    *data_items, buffer = buffer.split('\0')
-                                    for datum in data_items:
-                                        yield conversions.from_str(datum, api.return_type)
-                                else:
-                                    data = data.decode('utf-8')
-                                    yield conversions.from_str(data, api.return_type)
-            else:
-                url = f"{base_url}?self={self.self_id}"
-                kwargs_['self'] = self.self_id
-                payload = json.dumps({k: conversions.to_str(v) for k, v in kwargs_.items()})
-                async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
-                    async with session.post(url, data=payload) as resp:
-                        resp.raise_for_status()
-                        buffer = ""
-                        async for data, _ in resp.content.iter_chunks():
-                            if data:
-                                if api.return_type != bytes:
-                                    buffer += data.decode('utf-8')
-                                    *data_items, buffer = buffer.split('\0')
-                                    for datum in data_items:
-                                        yield conversions.from_str(datum, api.return_type)
-                                else:
-                                    data = data.decode('utf-8')
-                                    yield conversions.from_str(data, api.return_type)
+            try:
+                if rest_method == RestMethod.GET:
+                    url_args = cls._generate_url_args(self_id=self.self_id, kwargs=kwargs_)
+                    url = f"{base_url}{url_args}"
+                    async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
+                        async with session.get(url) as resp:
+                            if not resp.ok:
+                                sys.stderr.write((await resp.content.read()).decode('utf-8') + '\n')
+                            resp.raise_for_status()
+                            buffer = ""
+                            async for data, _ in resp.content.iter_chunks():
+                                if data:
+                                    if api.return_type != bytes:
+                                        buffer += data.decode('utf-8')
+                                        *data_items, buffer = buffer.split('\0')
+                                        for datum in data_items:
+                                            yield conversions.from_str(datum, api.return_type)
+                                    else:
+                                        data = data.decode('utf-8')
+                                        yield conversions.from_str(data, api.return_type)
+                else:
+                    url = f"{base_url}?self={self.self_id}"
+                    kwargs_['self'] = self.self_id
+                    payload = json.dumps({k: conversions.to_str(v) for k, v in kwargs_.items()})
+                    async with aiohttp.ClientSession(timeout=api.timeout, headers=common_headers) as session:
+                        async with session.post(url, data=payload) as resp:
+                            if not resp.ok:
+                                sys.stderr.write((await resp.content.read()).decode('utf-8') + '\n')
+                            resp.raise_for_status()
+                            buffer = ""
+                            async for data, _ in resp.content.iter_chunks():
+                                if data:
+                                    if api.return_type != bytes:
+                                        buffer += data.decode('utf-8')
+                                        *data_items, buffer = buffer.split('\0')
+                                        for datum in data_items:
+                                            yield conversions.from_str(datum, api.return_type)
+                                    else:
+                                        data = data.decode('utf-8')
+                                        yield conversions.from_str(data, api.return_type)
+            except aiohttp.ClientResponseError as e:
+                body = resp.content if resp is not None else "<<no response text/traceback info>>"
+                body += f"\n\nRequest to {api.name} failed: {e.message}"
+                raise Exception(body)
+
         setattr(clazz, name, instance_method_streamed)
 
     # noinspection PyPep8Naming
