@@ -2,8 +2,12 @@
 package for conversions to/from text or json
 """
 import dataclasses
+import datetime
 import json
+import types
 import typing
+import uuid
+from pathlib import Path
 from typing import Type, Any, Optional
 from enum import Enum
 assert typing
@@ -19,8 +23,12 @@ def normalize_to_json_compat(val: Any) -> Any:
                 json_data[key] = normalize_to_json_compat(json_data[key])
     elif isinstance(val, Enum):
         json_data = normalize_to_json_compat(val.value)
-    elif type(val) in (str, int, float, bool):
+    elif type(val) in (str, int, float, bool, Path):
         json_data = val
+    elif type(val) in (datetime.datetime, ):
+        json_data = val.isoformat()
+    elif type(val) in (uuid.UUID, ):
+        json_data = str(val)
     elif type(val) in [dict] or (getattr(type(val), '_name', None) in ('Dict', 'Mapping')):
         json_data = {}
         for key, value in val.items():
@@ -37,10 +45,30 @@ def normalize_to_json_compat(val: Any) -> Any:
 def normalize_from_json(json_data, typ) -> Any:
     if typ is None or json_data is None:
         return None
+    if isinstance(typ, types.UnionType):
+        for arg in typ.__args__:
+            if arg in (str, int, float, bool) and type(json_data) == arg:
+                return json_data
+            elif arg in (Path, ) and type(json_data) == str:
+                return Path(json_data)
+            elif json_data == '' and arg is types.NoneType:
+                return None
+            elif arg in (str, int, float, bool):  # json  data does not match this type
+                continue
+            # noinspection PyBroadException
+            try:
+                return normalize_from_json(json_data, arg)  # recurse on arg type
+            except Exception:
+                continue
+        raise TypeError(f"Cannot convert json data '{json_data}' to any of the Union types '{typ}'")
     if hasattr(typ, '_name') and (str(typ).startswith('typing.Union') or str(typ).startswith('typing.Optional')):
         for arg in typ.__args__:
             if arg in (str, int, float, ) and type(json_data) == arg:
                 return json_data
+            elif json_data == '' and arg is types.NoneType:
+                return None
+            elif json_data == '' and (arg is types.NoneType or arg is None):
+                return None
             elif arg in (str, int, float):
                 continue
             # noinspection PyBroadException
@@ -60,8 +88,12 @@ def normalize_from_json(json_data, typ) -> Any:
         return typ(json_data)
     elif typ == str:
         return json_data
-    elif typ in (int, float):
+    elif typ in (int, float, Path):
         return typ(json_data)
+    elif typ in (datetime.datetime, ):
+        return datetime.datetime.fromisoformat(json_data)
+    elif typ in (uuid.UUID, ):
+        return uuid.UUID(json_data)
     elif typ == bool:
         return str(json_data).lower() == 'true'
     elif getattr(typ, '_name', None) in ('Dict', 'Mapping'):
@@ -99,7 +131,11 @@ def to_str(val: Any) -> Optional[str]:
         return val.value
     elif type(val) == bool:
         return str(val).lower()
-    elif type(val) in (str, int, float):
+    elif type(val) in (str, int, float, Path):
+        return str(val)
+    elif type(val) in (datetime.datetime, ):
+        return val.isoformat()
+    elif type(val) in (uuid.UUID, ):
         return str(val)
     elif type(val) in [dict] or (getattr(type(val), '_name', None) in ('Dict', 'Mapping')):
         val = normalize_to_json_compat(val)
@@ -121,17 +157,35 @@ def _issubclass_safe(typ, clazz):
         return False
 
 
-def from_str(image: str, typ: Type) -> Any:
-    if hasattr(typ, '_name') and (str(typ).startswith('typing.Union') or str(typ).startswith('typing.Optional')):
+def from_str(image: str, typ: Type | types.UnionType) -> Any:
+    if (hasattr(typ, '_name') and (str(typ).startswith('typing.Union') or str(typ).startswith('typing.Optional')))\
+            or isinstance(typ, types.UnionType):
         # noinspection PyUnresolvedReferences
+        allow_none = str(typ).startswith('typing.Optional') or None in typ.__args__
+        if allow_none and not image:
+            # TODO: cannot really distinguish when return type is Optional[bytes] whether
+            #   None or bytes() should be returned
+            return None
+        # noinspection PyUnresolvedReferences
+        for arg in typ.__args__:
+            try:
+                if not image and arg is types.NoneType:
+                    return None
+                return arg(image)
+            except ValueError:
+                continue
         typ = typ.__args__[0]
     #######
     if _issubclass_safe(typ, Enum):
         return typ(image)
     elif typ == str:
         return image
-    elif typ in (int, float):
+    elif typ in (int, float, Path):
         return typ(image)
+    elif typ in (datetime.datetime, ):
+        return datetime.datetime.fromisoformat(image)
+    elif typ in (uuid.UUID,):
+        return uuid.UUID(image)
     elif typ == bool:
         return image.lower() == 'true'
     elif getattr(typ, '_name', None) in ('Dict', 'List', 'Mapping'):
