@@ -62,6 +62,24 @@ AsyncApi = Callable[['WebApplication', Request], Awaitable[StreamResponse]]
 PathLike = Union[Path, str]
 MAX_CLIENTS = 1024 ** 2
 
+# noinspection PyBroadException
+try:
+    # UGH!!! ths underlying low-level implementation swallows
+    # exceptions we need to process, so monkey patch :-(
+    # noinspection PyUnresolvedReferences,PyProtectedMember
+    from asyncio.selector_events import _SelectorTransport
+    # noinspection PyProtectedMember
+    _orig_fatal_error = _SelectorTransport._fatal_error
+
+    def _fatal_error_override(self, exc, msg):
+        _orig_fatal_error(self, exc, msg)
+        if isinstance(exc, ConnectionError):
+            raise
+
+    _SelectorTransport._fatal_error = _fatal_error_override
+except Exception:
+    log.warn(f"This platform may not support on_disconnect for simple requests")
+
 
 def wrap(app, op):
     async def wrapper(request):
@@ -69,6 +87,20 @@ def wrap(app, op):
     return wrapper
 
 
+async def response_from_exception(request, reason: str, code: int = 400):
+    text = traceback.format_exc()
+    resp = Response(status=code,
+                    reason=reason,
+                    text=text,
+                    content_type="text/plain"
+                    )
+    await resp.prepare(request)
+    return resp
+
+
+ASYNC_POLLING_INTERVAL = float(os.environ.get('BANTAM_ASYNC_POLL', 0.05))
+
+
 # noinspection PyUnresolvedReferences
 class WebApplication:
     """
@@ -115,7 +147,7 @@ class WebApplication:
                 await resp_q.put(resp)
 
             while True:
-                request = await cls.request_q.get(polling_interval=0.01)
+                request = await cls.request_q.get(polling_interval=ASYNC_POLLING_INTERVAL)
                 asyncio.create_task(task(*request))
 
     class ObjectRepo:
@@ -332,14 +364,24 @@ class WebApplication:
                             # handles case of _expire !-> expire
                             immediate._bantam_web_api._name = method._bantam_web_api.name
                         elif hasattr(method, '_bantam_web_api'):
+                            parameters1 = inspect.signature(method).parameters
+                            if 'cls' in parameters1:
+                                del parameters1['cls']
+                            parameters2 = inspect.signature(method).parameters
+                            if 'cls' in parameters1:
+                                del parameters2['cls']
                             has_diff_web_api = (
                                 method._bantam_web_api.method != immediate._bantam_web_api.method
                                 or method._bantam_web_api.content_type != immediate._bantam_web_api.content_type
                                 or method._bantam_web_api.is_constructor != immediate._bantam_web_api.is_constructor
+                                or (inspect.signature(method).return_annotation !=
+                                    inspect.signature(immediate).return_annotation)
+                                or parameters1 != parameters2
                             )
                             if has_diff_web_api:
                                 raise ValueError(
-                                    f"Mismatch in @web_api specifications in {class_name}.{method_name}"
+                                    f"Mismatch in @web_api specifications, parameter names/types or return type"
+                                    f"in {class_name}.{method_name}"
                                     f" and {ancestor.__name__}.{method_name}"
                                 )
                             elif method._bantam_web_api.arg_annotations != immediate._bantam_web_api.arg_annotations:
@@ -647,23 +689,44 @@ class WebApplication:
                         postprocess(response) if postprocess else response
                     except Exception as e:
                         text = traceback.format_exc()
-                        resp_ = Response(status=400, text=f"Error in post-processing of response: {e}\n{text}")
+                        resp_ = Response(status=400, reason=f"Exception in processing: {e}", text=text,
+                                         content_type="text/plain")
                         await resp_.prepare(request)
                         return resp_
                     return response
+                except PermissionError as e:
+                    return await response_from_exception(
+                        code=401,
+                        request=request,
+                        reason=f"PermissionError with request: {str(e)}"
+                    )
+                except TypeError as e:
+                    return await response_from_exception(
+                        code=400,
+                        request=request,
+                        reason=f"Improperly formatted query: {str(e)}"
+                    )
+                except HTTPException:
+                    raise
                 except (CancelledError, ConnectionResetError, ClientConnectionError):
                     raise
+                except Exception as e:
+                    return await response_from_exception(
+                        request,
+                        code=400,
+                        reason=f"General exception in request: {str(e)}"
+                    )
                 except BaseException as e:
-                    text = traceback.format_exc()
-                    resp_ = Response(status=500, reason=f"Server error: {e}",
-                                     body=f"Server error: {e}\n{text}", content_type="test/plain")
-                    await resp_.prepare(request)
-                    return resp_
+                    return await response_from_exception(
+                        request,
+                        code=500,
+                        reason=f"General exception when processing request: {str(e)}"
+                    )
             resp_q = asynciomultiplexer.AsyncAdaptorQueue(1)
             await cls.MainThread.start(invoke_proper(), resp_q)
             resp = await resp_q.get()
             if isinstance(resp, Exception):
-                raise
+                raise resp
             return resp
 
         invoke.clazz = WebApplication._instance_methods_class_map.get(api) if is_instance_method else None
@@ -746,11 +809,15 @@ class WebApplication:
                 # process the yielded value and allow execution to resume from yielding task
                 content_type = "text-streamed; charset=x-user-defined"
                 response = StreamResponse(status=200, reason='OK', headers={'Content-Type': content_type})
-                await response.prepare(request)
+                prepared = False
+                # iterate to get the one (and hopefully only) yielded element:
+                # noinspection PyTypeChecker
                 try:
-                    # iterate to get the one (and hopefully only) yielded element:
-                    # noinspection PyTypeChecker
                     async for res in result:
+                        if not prepared:
+                            await response.prepare(request)
+                            prepared = True
+                            # This is done post-await of first result in cas of exception right off the bat
                         try:
                             serialized = _serialize_return_value(res, encoding)
                             if not isinstance(res, bytes):
@@ -758,19 +825,26 @@ class WebApplication:
                             await response.write(serialized)
                         except (CancelledError, ConnectionResetError, ClientConnectionError):
                             if api.on_disconnect is not None:
-                                if inspect.iscoroutinefunction(api.on_disconnect):
+                                if inspect.iscoroutinefunction(api.on_disconnect) or\
+                                    (hasattr(api.on_disconnect, '__func__') and
+                                     inspect.iscoroutinefunction(api.on_disconnect.__func__)):
                                     await api.on_disconnect(res)
                                 else:
                                     api.on_disconnect(res)
                             raise
-
-                except (CancelledError, ConnectionResetError, ClientConnectionError):
+                except (CancelledError, ConnectionError, ClientConnectionError):
                     raise
                 except Exception as e:
-                    print(str(e))
-                    raise asyncio.CancelledError(f"Error in server-side logic: {e}") from e
-                await response.write_eof()
-                return response
+                    if not prepared:
+                        response.set_status(400, reason=f"Exception in request: {str(e)}")
+                        await response.prepare(request)
+                    else:
+                        log.error(f"Exception in server-side logic handling request: {str(e)}")
+                finally:
+                    if not prepared:  # nothing generated in this case, but still a 200
+                        await response.prepare(request)
+                    await response.write_eof()
+                    return response
             else:
                 #################
                 #  regular response
@@ -797,30 +871,18 @@ class WebApplication:
                 resp = Response(status=200, body=result if result is not None else b"Success",
                                 content_type=content_type)
                 await resp.prepare(request)
+                try:
+                    await resp.write_eof()
+                except ConnectionError:
+                    if api.on_disconnect is not None:
+                        if inspect.iscoroutinefunction(api.on_disconnect) or \
+                                (hasattr(api.on_disconnect, '__func__') and
+                                 inspect.iscoroutinefunction(api.on_disconnect.__func__)):
+                            await api.on_disconnect(result)
+                        else:
+                            api.on_disconnect(result)
+                    raise
                 return resp
-        except PermissionError as e:
-            resp = Response(status=401,
-                            reason=f"Improperly formatted query: {str(e)}\n{traceback.format_exc()}",
-                            text=f"Improperly formatted query: {str(e)}\n{traceback.format_exc()}",
-                            )
-            await resp.prepare(request)
-            return resp
-        except TypeError as e:
-            resp = Response(status=400,
-                            reason=f"Improperly formatted query: {str(e)}\n{traceback.format_exc()}",
-                            text=f"Improperly formatted query: {str(e)}\n{traceback.format_exc()}",
-                            )
-            await resp.prepare(request)
-            return resp
-        except HTTPException:
-            raise
-        except Exception as e:
-            resp = Response(status=400,
-                            reason=f"Exception: {e}: \n{traceback.format_exc()}",
-                            text=f"Exception: {e}: \n{traceback.format_exc()}",
-                            )
-            await resp.prepare(request)
-            return resp
         finally:
             # noinspection PyUnresolvedReferences,PyProtectedMember
             del cls._context[sys._getframe(0)]
@@ -930,17 +992,17 @@ class WebApplication:
                 awaitable = api(api.clazz, *varargs, **kwargs)
             else:
                 kwargs, varargs = api.update_varargs(kwargs)
-                awaitable = api(*varags, **kwargs)
+                awaitable = api(*varargs, **kwargs)
             if inspect.isasyncgen(awaitable):
                 #################
                 #  streamed response through async generator:
                 #################
                 # underlying function has yielded a result rather than turning
                 # process the yielded value and allow execution to resume from yielding task
-                async_q = asyncio.Queue()
+                # async_q = asyncio.Queue()
                 content_type = "text/streamed; charset=x-user-defined"
                 response = StreamResponse(status=200, reason='OK', headers={'Content-Type': content_type})
-                await response.prepare(request)
+                prepared = False
                 try:
                     # iterate to get the one (and hopefully only) yielded element:
                     # noinspection PyTypeChecker
@@ -950,28 +1012,41 @@ class WebApplication:
                             serialized = _serialize_return_value(res, encoding)
                             if not isinstance(res, bytes):
                                 serialized += b'\0'
+                            if not prepared:
+                                await response.prepare(request)
                             await response.write(serialized)
                             count += 1
                         except (CancelledError, ConnectionResetError, ClientConnectionError):
                             if api.on_disconnect is not None:
-                                # noinspection PyUnboundLocalVariable
-                                args = (instance, res, ) if api.is_instance_method else (res, )
-                                if inspect.iscoroutinefunction(api.on_disconnect):
-                                    await api.on_disconnect(*args)
-                                else:
-                                    api.on_disconnect(*args)
-                            raise
-
+                                try:
+                                    # noinspection PyUnboundLocalVariable
+                                    args = (instance, res, ) if api.is_instance_method else (res, )
+                                    if inspect.iscoroutinefunction(api.on_disconnect) or\
+                                        (hasattr(api.on_disconnect, '__func__') and
+                                         inspect.iscoroutinefunction(api.on_disconnect.__func__)):
+                                        await api.on_disconnect(*args)
+                                    else:
+                                        api.on_disconnect(*args)
+                                except Exception as e:
+                                    print(f"ERROR in call to on_disconnect from bantam async generator: {e}")
+                            break
                 except (CancelledError, ConnectionResetError, ClientConnectionError):
                     raise
                 except Exception as e:
-                    print(str(e))
-                    await async_q.put(None)
-                    resp = Response(status=500, body=f"Exception in server-side logic: {e}",
-                                    content_type='text/plain')
-                    await resp.prepare(request)
-                    return resp
-                await response.write_eof()
+                    logging.error(str(e))
+                    if not prepared:
+                        response = Response(status=400, reason=f"Exception in servicing request: {e}",
+                                            body=f"Exception in servicing request: {e}")
+                        await response.prepare(request)
+                    else:
+                        await response.write(f"Exception in server-side logic: {e}".encode('utf-8'))
+                        with suppress(Exception):
+                            await response.write_eof()
+                        return response
+                finally:
+                    if not prepared:
+                        await response.prepare(request)
+                    await response.write_eof()
                 return response
             else:
                 #################
@@ -1003,22 +1078,6 @@ class WebApplication:
                                 content_type=content_type)
                 await resp.prepare(request)
                 return resp
-
-        except TypeError as e:
-            resp = Response(status=400, text=f"Improperly formatted query: {str(e)}")
-            await resp.prepare(request)
-            return resp
-        except HTTPException as e:
-            resp = Response(status=e.status_code, text=str(e))
-            await resp.prepare(request)
-            return resp
-        except (CancelledError, ConnectionResetError, ClientConnectionError):
-            raise
-        except Exception as e:
-            text = f"Exception: {e}\n {traceback.format_exc()}"
-            resp = Response(status=500, text=text)
-            await resp.prepare(request)
-            return resp
         finally:
             # noinspection PyUnresolvedReferences,PyProtectedMember
             del cls._context[sys._getframe(0)]
